import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/const.dart';
import 'package:flutter_custom_form_component/widget/field/bank_card_number/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/box_decorations.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/widget.dart';
import 'package:flutter_custom_form_component/widget/field/text_styles.dart';

/// Text field supports bank card number format.
class BankCardNumberFieldWidget extends OneValueFieldWidget {
  late final String _separator;

  BankCardNumberFieldWidget(
      {final TextInputAction action = fieldAction,
      final BoxDecorations? decorations,
      final bool enabled = fieldEnabled,
      final String? hint,
      final double horizontalPadding = fieldHorizontalPadding,
      final String? initialValue,
      final Key? key,
      final Function(dynamic value)? onChanged,
      final String separator = '-',
      final TextStyles? styles})
      : super(
            action: action,
            decorations: decorations,
            enabled: enabled,
            hint: hint,
            horizontalPadding: horizontalPadding,
            initialValue: initialValue,
            key: key,
            onChanged: onChanged,
            styles: styles) {
    _separator = separator;
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<BankCardNumberFieldWidget>
    with FieldMixin, OneValueFieldMixin {
  final FocusNode _focusNode = FocusNode();
  late final BankCardNumberFormatter _formatter;
  late final int _maxLength;

  @override
  Widget build(final BuildContext context) => container(TextField(
      controller: controller,
      decoration: InputDecoration(
          border: InputBorder.none,
          counterText: '',
          hintStyle: widget.styles.hint,
          hintText: widget.hint),
      enabled: widget.enabled,
      focusNode: _focusNode,
      inputFormatters: [FilteringTextInputFormatter.digitsOnly, _formatter],
      keyboardType: TextInputType.number,
      maxLength: _maxLength,
      style: style,
      textInputAction: widget.action));

  @override
  void controllerListener({final Function(dynamic value)? onChanged}) {
    if (!valid && controller.text.length == _maxLength) {
      valid = true;
      setState(() {});
      Future.delayed(const Duration(milliseconds: 200),
          () => super.controllerListener(onChanged: onChanged));
    } else {
      super.controllerListener(onChanged: onChanged);
    }
  }

  @override
  void dispose() {
    _focusNode.removeListener(_focusNodeListener);
    _focusNode.dispose();
    disposeMixin();
    super.dispose();
  }

  /// Listener added to the focus node.
  void _focusNodeListener() {
    if (!_focusNode.hasFocus) {
      if (controller.text.length < _maxLength) {
        valid = controller.text.isEmpty;
      } else {
        valid = controller.text.length == _maxLength;
      }
      setState(() {});
    }
  }

  @override
  void initState() {
    super.initState();
    _formatter = BankCardNumberFormatter(widget._separator);

    /// 16 digits and 3 separators between.
    _maxLength = 16 + widget._separator.length * 3;

    if (widget.initialValue != null) {
      valid = widget.initialValue!.toString().length < 16;
    }

    initStateMixin(
        initialValue: widget.initialValue, onChanged: widget.onChanged);
    _focusNode.addListener(_focusNodeListener);
  }

  @override
  BoxDecoration get decoration {
    if (_focusNode.hasFocus) {
      return widget.decorations.active;
    }
    if (valid) {
      return widget.decorations.box;
    }
    return widget.decorations.error;
  }

  @override
  double get horizontalPadding => widget.horizontalPadding;

  @override
  TextStyle get style {
    if (_focusNode.hasFocus) {
      return widget.styles.active;
    }
    if (valid) {
      return widget.styles.text;
    }
    return widget.styles.error;
  }

  @override
  String get value => controller.text.replaceAll(widget._separator, '');
}
