import 'package:flutter/material.dart';
import 'package:flutter_custom_form_component/widget/const.dart';
import 'package:flutter_custom_form_component/widget/field/box_decorations.dart';
import 'package:flutter_custom_form_component/widget/field/date/422/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/date/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/text_styles.dart';
import 'package:flutter_custom_form_component/widget/field/widget.dart';

/// Text field supports YYYYDDMM date format and validation.
/// For another date format can refer to DdMmYyyyFieldWidget, MmDdYyyyFieldWidget or YyyyMmDdFieldWidget.
class YyyyDdMmFieldWidget extends FieldWidget {
  late final int? _initialYear, _initialMonth, _initialDay;
  late final bool _initialValid;
  late final Function(int? year, int? month, int? day)? _onChanged;
  late final String _separator;

  YyyyDdMmFieldWidget(
      {final TextInputAction action = fieldAction,
      final BoxDecorations? decorations,
      final bool enabled = fieldEnabled,
      final String hint = 'yyyy - dd - mm',
      final double horizontalPadding = fieldHorizontalPadding,
      final int? initialYear,
      final int? initialMonth,
      final int? initialDay,
      final Key? key,
      final Function(int? year, int? month, int? day)? onChanged,
      final String separator = ' - ',
      final TextStyles? styles,
      final bool valid = fieldValid})
      : super(
            action: action,
            decorations: decorations,
            enabled: enabled,
            hint: hint,
            horizontalPadding: horizontalPadding,
            key: key,
            styles: styles) {
    _initialYear = initialYear;
    _initialMonth = initialMonth;
    _initialDay = initialDay;
    _initialValid = valid;
    _onChanged = onChanged;
    _separator = separator;
  }

  @override
  State<StatefulWidget> createState() => _State();
}

/// State of YyyyMmDdFieldWidget.
class _State extends State<YyyyDdMmFieldWidget>
    with DateFieldStateMixin, FieldMixin {
  @override
  Widget build(final BuildContext context) => buildMixin(
      action: widget.action,
      enabled: widget.enabled,
      hint: widget.hint,
      styles: widget.styles);

  @override
  void dispose() {
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    String? initialYear, initialMonth, initialDay, initialDate;
    if (widget._initialYear != null) {
      initialYear = formatter.completeDigits(widget._initialYear!, digits: 4);
    }
    if (widget._initialMonth != null) {
      initialMonth = formatter.completeDigits(widget._initialMonth!, digits: 2);
    }
    if (widget._initialDay != null) {
      initialDay = formatter.completeDigits(widget._initialDay!, digits: 2);
    }
    if (initialYear != null && initialMonth != null && initialDay != null) {
      initialDate = formatter.format('$initialYear$initialDay$initialMonth');
    }
    initStateMixin(
        initialDate: initialDate,
        initialValid: widget._initialValid,
        separator: widget._separator);
  }

  /// Obtaining the day, month and year. Then calling external validation (widget._onChanged).
  @override
  void onChanged() {
    final List<String> strings = split(separator: widget._separator);
    year = int.tryParse(strings.isNotEmpty ? strings[0] : '');
    month = int.tryParse(strings.length > 2 ? strings[2] : '');
    day = int.tryParse(strings.length > 1 ? strings[1] : '');
    widget._onChanged?.call(year, month, day);
  }

  /// Refreshing the UI by calling setState().
  @override
  void refresh() => setState(() {});

  @override
  Date422Formatter get formatter => Date422Formatter(widget._separator);

  @override
  BoxDecoration get decoration {
    if (hasFocus()) {
      return widget.decorations.active;
    }
    if (valid) {
      return widget.decorations.box;
    }
    return widget.decorations.error;
  }

  @override
  double get horizontalPadding => widget.horizontalPadding;

  @override
  TextStyle get style {
    if (hasFocus()) {
      return widget.styles.active;
    }
    if (valid) {
      return widget.styles.text;
    }
    return widget.styles.error;
  }
}
